/*Funktionen die in irgend einwer Weise mit menschlicher
  Interaktion zu tun haben
*/

#include "main.h"

u08 volatile leds;
u08 volatile taster;
u08 volatile tast_wert;
u08 volatile tast_wert_max;
char cmdline[13];
u08 cmdidx;

void timer0_init(void) {
TCNT0 = 0;  //Counter reseten
TIMSK |= 1<<TOIE0;//Timer0 Overflow Int Aktivieren
TCCR0 = 5; //Prescaler: 1024
}

SIGNAL(SIG_OVERFLOW0) {
//Der Interrupt wird 122mal pro Sekunde aufgerufen
u08 leds_l; //wegen optimierung der volatile
u08 tast_wert_l; //wegen optimierung der volatile
u08 tast_wert_max_l;
u08 temp;
u08 PORTX;
/* PORTX wird verwendet, damit die LEDs nicht unntig
flackern (insbesondere beim Unterbrechen durch andere
Interrupts) und so unntige Strimpulse erzeugen.
Nebenbei spart die Verwendung von PORTX auch noch
32 Byte Flash.
*/
//wir lassen andere Interrupts in dieser routine zu!
sei();
TCNT0 = 0xff -244; //Die Routine wird 32mal pro Sec durchlaufen
//LED Anzeige steuern,
leds_l = ~leds;
//Alle LEDs erst einmal aus
//Port C
PORTX = PORTC & 0xcf;
//LED 1
PORTX |= (u08)((leds_l & 0x01)<<5);
//LED 2
PORTX |= (u08)(((leds_l >> 1) & 0x01)<<4);
PORTC = PORTX;
//Port B
PORTX = PORTB & 0xf8;
//LED 3
PORTX |= (u08)(((leds_l >> 2) & 0x01)<<2);
//LED 4
PORTX |= (u08)(((leds_l >> 3) & 0x01)<<1);
//LED 5
PORTX |= (u08)(((leds_l >> 4) & 0x01)<<0);
PORTB = PORTX;
//Port D
PORTX = PORTD & 0xc7;
//LED 6
PORTX |= (u08)(((leds_l >> 5) & 0x01)<<5);
//LED 7
PORTX |= (u08)(((leds_l >> 6) & 0x01)<<4);
//LED 8
PORTX |= (u08)(((leds_l >> 7) & 0x01)<<3);
PORTD = PORTX;
//Taster debounce (low-aktiv)
tast_wert_l = tast_wert;
if (tast_pin != 0) { //losgelassen
  tast_wert_l >>= 1;
} else {
  if (tast_wert_l < 255) {
    tast_wert_l++;
  }
}
tast_wert_max_l = tast_wert_max;
if (tast_wert_l > tast_wert_max_l) {
  tast_wert_max_l = tast_wert_l;
}
if ((tast_wert_l == 0) && (tast_wert_max_l != 0)) {
  //Wenn mindestens 100ms gedrckt:
  if (tast_wert_max_l > 3) taster = 1; //Funktion 1
  //Wenn mind ca 0,5s gedrckt
  if (tast_wert_max_l > 16) taster = 2; //funktion 2
  //Wenn mehr als 2sec
  if (tast_wert_max_l > 64) taster = 3; //zu lange
  tast_wert_max_l = 0;
}
tast_wert = tast_wert_l;
tast_wert_max = tast_wert_max_l;
//Pings
temp = ping_next;
if (temp > 0) {
  temp--;
  ping_next = temp;
}
//Connected
temp = connected_rx;
if (temp > 0) {
  temp--;
  connected_rx = temp;
}
temp = connected_tx;
if (temp > 0) {
  temp--;
  connected_tx = temp;
}
//nosend
temp = nosend;
if ((temp > 0)&&(temp != 255)) {
  temp--;
  nosend = temp;
}
}

void taster_act(void) {
u08 taster_l;
/* wir wollen die Mglkichkeit ausschlieen, dass
zwischen dem auslesen von taster und dem zurcksetzen
genau dann neue Werte in taster geschieben werden,
da sonst ein Tastendruck verloren gehen knnte
*/
TIMSK &= ~(1<<TOIE0); //Timer0 interrupt stoppen
taster_l = taster;
taster = 0;
TIMSK |= 1<<TOIE0; //Timer0 interrupt wieder altivieren
if (taster_l == 1) { //Mastermode on/off
  send_funk_put(0x0120|mastermode);
  mastermode = 1 - mastermode;
}
if (taster_l == 2) { //interface on/off
  interface_active = 1 - interface_active;
}
if (taster_l == 3) { //zu lang, stoppe alles
  error(1);
}
}

void led_update(void) {
u08 leds_l;
//Alle LEDs werden von dieser Routine aktualisiert
leds_l = 0;
//Buffer frei < 50%
if (rec_232_free < (rec_232_size/2)) {
  leds_l |= 1<<5;
}
//Wenn Buffer voll
if (rec_232_status() == 2) {
  leds_l |= 1<<4;
}
//Wenn Master
if (mastermode == 1) {
  leds_l |= 1<<1;
}
//Wenn Interface Activ
if (interface_active == 1) {
  leds_l |= 1<<0;
}
//Funk hat Daten empfangen
if (rec_funk_status() != 0) {
  leds_l |= 1<<6;
}
//Funk soll Daten senden
if (send_funk_status() != 0) {
  leds_l |= 1<<7;
}
//Verbindung RX steht
if (connected_rx > 0) {
  leds_l |= 1<<3;
}
//Verbindung TX steht
if (connected_tx > 0) {
  leds_l |= 1<<2;
}
leds = leds_l;
}

void interface(void) {
/* Befehle
$funk xyz
setzt srspeed auf xyz (nur eeprom)
$funk
gibt aktuelles srspeed aus
$baud xyz
berechnet und setzt die uart Baudrate (nur eeprom)
$baud
gibt die aktuelle Baudrate aus
$reboot
resetet die Schaltung per Watchdog
$ping
sendet Ping per Funk
$AD x
gibt den A/D Wert von x aus
$output x
setzt x als Ausgang
$input x
setzt x als Eingang
$set x
setzt x auf High
$reset x
setzt x auf Low
$get x
gibt den digitalen Zustand des Pins zurck
$master x
setzt mastermode (nur eeprom)
$intact x
setzt interface aktive (nur eeprom)
$intsrc x
setzt interface source (nur eeprom)
$retry xyz
wie oft das senden per Funk versucht werden soll (nur eeprom)
$retry
gibt send_funk_retry zurck
$halt x
setzt halt_on_error (nur eeprom)
*/

/* als Erstes wurd ein Commandbyte aus der richtigen Quelle genommen */
u16 val2;
u08 cmd = 0;
if (rec_funk_status() != 0) { //Funk hat was empfangen
  val2 = rec_funk_get();
  connected_rx = 160; //fr 5 sec
  if (val2 > 0xFF) {
    syscommand((u08)val2);
  } else {
    if (interface_source == 1) {
      cmd = (u08)val2;
    }
  }
}
if ((interface_source == 0) && (rec_232_status() != 0)) {
  cmd = rec_232_get();
}
if (cmd != 0) { //0: kein Befehlt oder eh ungltig
  if (cmd == 13) {//Enter gedrckt, Befehl auswerten!
    cmd_execute();
  } else {
    /*zunchst wird der Befehl in nem Array eingefangen
    alles restliche was kein Systembefehl ist wird verworfen */
    //Backspace bearbeiten
    if (cmd == 8) { //Backspace
      if (cmdidx > 0) { //Und mindestens ein Zeichen bereits eingegeben
        cmdidx--;
        cmdline[cmdidx] = ' '; //Altes Zeichen durch Leerzeichen ersetzen
      } else { //Noch kein Zeiche was gelscht werden kann
        cmd = 0; //kein Echo
      }
    } else //Kein Backspace
    if (cmdidx < 12) { //wenn cmdline nicht voll
      cmdline[cmdidx] = cmd;
      cmdidx++;
    } else { //cmdline wrde berlaufen
      cmd = 0;
    }
    if (cmd != 0) { //wenn nicht 0 Zeichen, dann Echo
      if (interface_source == 0) { //Echo des Zeichens senden
        send_232_put(cmd);
      } else {
        send_funk_put(cmd);
      }
    }
  }
}

}

const char cmd_list[cmd_count_max][7] PROGMEM = {
{"FUNK  "},
{"BAUD  "},
{"REBOOT"},
{"PING  "},
{"AD    "},
{"OUTPUT"},
{"INPUT "},
{"SET   "},
{"RESET "},
{"MASTER"},
{"INTACT"},
{"INTSRC"},
{"RETRY "},
{"HALT  "},
{"GET   "},
{"HELP  "},
{"ABOUT "}
};

const s16 cmd_param_range[cmd_count_max][2] PROGMEM = {
{10,80},
{2400,19200},
{1,0},
{1,0},
{1,4},
{1,4},
{1,4},
{1,4},
{1,4},
{0,1},
{0,1},
{0,1},
{0,255},
{0,1},
{1,4},
{1,0},
{1,0}
};



const char cmd_answer[7][14] PROGMEM = {
{"\n\rCmd unknown"},
{"\n\rParam wrong"},
{"\n\rNeed reboot"},
{"\n\rDone       "},
{"\n\ris:        "},
{"\n\rSee source "},
{"\n\rby Malte   "}
};

void cmd_execute(void) {
s16 cmd_param = -1;
u08 cmd_param08;
u08 cmd_param_valid = 0;
u08 cmd_token = 0;
u08 str_par = 0;
u08 nun;
u16 tmp16;
u08 tmp08;
char cmd_answer_send[17] = "\n\rNeed debug ";
//erstmal alles in Grobuchstaben konvertieren
strupr(cmdline);
//Parameter erkennen + mit whitespaces berschreiben
for (nun = 0;nun < 10;nun++) {
  if (str_par == 1) { //Parameter
    if ((cmdline[nun] < 58) && (cmdline[nun] > 47)) {
      if (cmd_param < 0) { //um das -1 da wegzubekommen
        cmd_param = 0;
      }
      cmd_param = cmd_param*10 +(cmdline[nun] - 48);
    }
  }
  if (cmdline[nun] == ' ') { //beginn des Parameters
    str_par = 1;
  }
  if ((cmdline[nun] < 'A') || (cmdline[nun]  > 'Z')) { //alles was keine Buchstaben ist...
    cmdline[nun] = ' '; //...mit Leerzeichen berschreiben
  }
}
//Token erkennen
for (cmd_token = 0;cmd_token < cmd_count_max;cmd_token++) {
  if (strncmp_P(cmdline,cmd_list[cmd_token],6) == 0) { //wenn Befehl gleich dem in der Liste
    break;
  }
}
//Rangecheck des Parameters
if (cmd_token != cmd_count_max) {
  strcpy_P(cmd_answer_send,cmd_answer[1]); //Default Anwort: falscher Parameter
  if (cmd_param == -1)  { //kein Parameter vorhanden
    cmd_param_valid = 0;
  }else{
    if ((pgm_read_word(&cmd_param_range[cmd_token][0]) <= cmd_param) &&
       (pgm_read_word(&cmd_param_range[cmd_token][1]) >= cmd_param)) {
      cmd_param_valid = 1; //alles ok
    } else {
      cmd_param_valid = 2; //fehlerhaft
    }
  }
}
cmd_param08 = (u08)cmd_param; //oft ist der PArameter nur ein 8Bit Wert -> winiger Code
//Eigentliche Reaktion auf den Befehl
if (cmd_param_valid != 2) {
  if (cmd_token == 0) { //FUNK
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_srspeed[0],cmd_param08);
      eeprom_write_byte(&eep_srspeed[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen Wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,srspeed,3);
    }
  }
  if (cmd_token == 1) { //BAUD
    if (cmd_param_valid == 1) {
      tmp08 = (F_CPU/(16*cmd_param))-1;
      eeprom_write_byte(&eep_rs232ubrr[0],tmp08);
      eeprom_write_byte(&eep_rs232ubrr[1],tmp08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else {
      tmp16 = F_CPU/(16*(rs232ubrr+1));
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,tmp16,6);
    }
  }
  if (cmd_token == 2) { //REBOOT
    WDTCR = (1<<WDE) + 7; //Watchdog enabled, lngste dauer, um eventuell noch Done zu senden
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
  if (cmd_token == 3) { //PING
    send_funk_put(0x0110); //Sende Ping
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
  if (cmd_token == 9) { //MASTER
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_mastermode[0],cmd_param08);
      eeprom_write_byte(&eep_mastermode[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,mastermode,1);
    }
  }
  if (cmd_token == 10) { //INTACT
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_interface_active[0],cmd_param08);
      eeprom_write_byte(&eep_interface_active[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,interface_active,1);
    }
  }
  if (cmd_token == 11) { //INTSRC
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_interface_source[0],cmd_param08);
      eeprom_write_byte(&eep_interface_source[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,interface_source,1);
    }
  }
  if (cmd_token == 12) { //RETRY
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_send_funk_retry[0],cmd_param08);
      eeprom_write_byte(&eep_send_funk_retry[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,send_funk_retry,3);
    }
  }
  if (cmd_token == 13) { //HALT
    if (cmd_param_valid == 1) {
      eeprom_write_byte(&eep_halt_on_err[0],cmd_param08);
      eeprom_write_byte(&eep_halt_on_err[1],cmd_param08);
      strcpy_P(cmd_answer_send,cmd_answer[2]);
    } else { //aktuellen wert von srspeed ausgeben
      strcpy_P(cmd_answer_send,cmd_answer[4]);
      wordtostr(cmd_answer_send,halt_on_err,1);
    }
  }
  if (cmd_token == 14) { //GET
    strcpy_P(cmd_answer_send,cmd_answer[4]);
    if (cmd_param_valid == 1) {
      wordtostr(cmd_answer_send,(PINC>>(cmd_param08-1))&1,1);
    } else { //aktuellen Wert von srspeed ausgeben
      wordtostr(cmd_answer_send,(PINC & 0x0F),2);
    }
  }
  if (cmd_token == 15) { //Hilfe
    strcpy_P(cmd_answer_send,cmd_answer[5]);
  }
  if (cmd_token == 16) { //About
    strcpy_P(cmd_answer_send,cmd_answer[6]);
  }
}
if (cmd_param_valid == 1) { //nur mit Parameter berhaupt gltig
  if (cmd_token == 4) { //AD
    tmp16 = getadc(cmd_param-1);
    strcpy_P(cmd_answer_send,cmd_answer[4]);
    wordtostr(cmd_answer_send,tmp16,5);
  }
  if (cmd_token == 5) { //OUTPUT
    DDRC |= 1<<(cmd_param-1);
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
  if (cmd_token == 6) { //INPUT
    DDRC &= ~(1<<(cmd_param-1));
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
  if (cmd_token == 7) { //SET
    PORTC |= 1<<(cmd_param-1);
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
  if (cmd_token == 8) { //RESET
    PORTC &= ~(1<<(cmd_param-1));
    strcpy_P(cmd_answer_send,cmd_answer[3]);
  }
}
if (cmd_token == cmd_count_max) { // Befehl unbekannt
  strcpy_P(cmd_answer_send,cmd_answer[0]);
}
//Linefeed, Carriage Return und $ Bytes anhngen
cmd_answer_send[14] = '\r'; //Carriage Return
cmd_answer_send[15] = '\n'; //Linefeed
cmd_answer_send[16] = '$';  //$
//Ausgabe in entsprechenden FIFO kpoieren
for (nun = 0;nun < 17;nun++) {
  if (interface_source == 0) { // 0 = RS232
    send_232_put(cmd_answer_send[nun]);
  }else{
    send_funk_put(cmd_answer_send[nun]);
  }
}
//fr nchsten Befehlt zurck auf Anfang
for (cmdidx = 11;cmdidx != 0;cmdidx--) {
  cmdline[cmdidx] = ' ';
}
}

void wordtostr(char *s, u16 nummer, u08 digits) {
s += 6;
while (digits > 0) {
digits--;
*(s+digits) = nummer%10+48;
nummer = nummer / 10;
}
}
