/* Datei Beschreibung
  Titel:    Funk bertragung
  Autor:    (c) 2005 by Malte Marwedel
  Datum:    2005-03-18
  Version:  Beta 1
  Zweck:    bertragung von Daten via Funk
  Software: AVR-GCC
  Hardware: Funk Empfnger/Sender, ATMEGA8 mit 8MHZ
  Bemerkung:Dieses Projekt darf nach belieben compiliert und verndert
            werden. Die Verwendung des Codes geschieht jedoch auf eigene
            Verantwortung, eine Fehlerfreiheit wird nicht garantiert.
            Bei nderungen muss kenntlich gemacht werden, dass es sich nicht
            mehr um die original Dateien handelt und der Hinweis
            erhalten bleibt, dass von mir die ursprngliche Version stammt.
  Wer Fragen oder Anregungen zu dem Programm hat, kann an
             m.marwedel <AT> onlinehome.de mailen.
            Mehr ber Elektronik und AVRs gibt es auf meiner Homepage:
             http://www.marwedels.de/malte/
  Code Gre:ATMEGA8 : 5342Byte    

  
Pinbelegung der Schaltung:
PB0: Output, LED 5 (Buffer Full)
PB1: Output, LED 4 (Verbindung RX)
PB2: Output, LED 3 (Verbindung TX)
PB3: Input + Pull-up,  Taster/MOSI
PB4: Input,  MISO
PB5: Input,  SCK
PB6: Input,  nicht vorhanden
PB7: Input,  nicht vorhanden

PC0: Input,  User I/O 1
PC1: Input,  User I/O 2
PC2: Input,  User I/O 3
PC3: Input,  User I/O 4
PC4: Output, LED 2 (Master)
PC5: Output, LED 1 (Interface Active)
PC6: Input,  nicht vorhanden
PC7: Input,  nicht vorhanden

PD0: Input,  RXD
PD1: Output, TXD
PD2: Output, 433MHZ Sender
PD3: Output, LED 8 (Send)
PD4: Output, LED 7 (Recive)
PD5: Output, LED 6 (Buffer > 50%)
PD6: Input,  433MHZ Empfnger (Analog Comparator)
PD7: Input,  Referenz (Analog Comparator)

Funk-Systembefehle:
0x0F: Wird beim Power Up gesendet und muss selber empfangen werden knnen
0x10: "Ping" senden
0x11: Auf "Ping" antworten
0x12: Antwort auf Ping Antwort (nur Master sendet diese)
0x20: Setze auf Slave
0x21: Setze auf Master

Fehlercode der LEDs (Blinken)
Falls die LEDs Blinken, so ist ein Fehler aufgetreten und der Controller hat alle
anderen arbeiten gestoppt. Die LEDs zeigen eine Zahl (binr) an, die auf die
Ursache des fehlers deutet:
Fehlercodes:
1: Die Taste wurde zu lange gedrckt (ungltig)
   Lsung: Taste rechtzeitig loslassen oder Anschlsse berprfen
2: Es kam zu unstimmigkeitem beim Lesen des EEproms
   Lsung: Default Daten wiederherstellen
->Fehlercode 3 - 11 werden nur bei halt_on_err != 0 angezeigt!<-
3: Selbsttest des Senders schlug fehl
   Lsung: 1. Mehrfach erneut versuchen
           2. Default Daten im EEprom wiederherstellen
           3. Schaltung an einem strungsfreiem Ort erneut testen
           4. Sender + Empfnger, Leitungen und Betriebsspannung berprfen
4: Buffer berlauf von rec_funk
5: Buffer berlauf von send_232
7: Buffer berlauf von send_funk
8: Buffer Leerlauf von rec_funk
9: Buffer Leerlauf von send_232
10: Buffer Leerlauf von rec_232
11: Buffer Leerlauf von send_funk

        
Funktionsweise des Tasters:
1. Whrend der Initialisierung (alle LEDs leuchten):
  0,5 - 2,0sec drcken: Wiederherstellen der Default Einstellungen im EEProm.
                        Bei Erfolg zeigen die LEDs ein kurzes Lauflicht.
2. Im normalem Betrieb:
  0,1 - 0,5sec drcken: Master/Slave umschalten
  0,5 - 2,0sec drcken: Interface an/aus schalten
  ab 2sec      drcken: Fehlercode 1
3. Bei Fehler (blinkende LEDs):
  0,5 - 2,0sec drcken: Neustart des Controllers (Reset durch Watchdog)

Befehle im Interface Modus:
Die Liste der Befehle befindet sich ungefhr in der Mitte
in der Datei "human-io.c" im cmd_list array.
Im cmd_param_range arry sind die min (erster Wert) und max (zweiter) Wert fr
eventuelle Parameter angegeben. Ist der min wert grer als der Max Wert, so ist
kein Parameter erlaubt. Die Parameter werden durch ein Leerzeichen vom Befehl getrennt
Bei der Antwort "Need Reboot" wurden die werte ins EEProm geschrieben und werden
erst bei einem Reboot aktiv.
ACTUNG: Bei der Eingabe der Baudrate fr die Schnittstelle wird nicht berprft
ob auch die Abweichung von der Soll-Baudrate innerhalb des Erlaubten liegt.
Nhere Infos hierzu (Berechnen des Fehlers in % u.s.w.) gibt das Datenblatt des
Controllers.

Version History (nur wesentliche nderungen sind aufgelistet):
Test1: Funktionstest der Schaltung (richtig verltet u..)
Alpha 1: 2926Byte, Funkbertragung mglich
Alpha 2: 3204Byte, weniger Debugmeldungen, auto Ping, LEDs
         Datenbergabe zwischen FIFOs, Sytem Befehle
Alpha 3: 3376Byte, Master/Slave verhalten
Alpha 4: 5266Byte, Interface Modus, etwas Doku
Beta 1: 5342Byte, halt_on_err -> damit alle geplanten Funktionen implementiert
*/


#include "main.h"

//Variablen fr die Konfiguration des Programms
u08 mastermode;
u08 rs232ubrr;
u08 volatile srspeed;
u08 volatile srspeed_3_2;
u08 volatile srspeed_5_2;
u08 interface_active;
u08 interface_source;
u08 halt_on_err;
u08 send_funk_retry;
u08 volatile ping_next;
u08 volatile connected_rx;
u08 volatile connected_tx;
u08 volatile nosend;


void waitms(uint16_t zeitms) {
//50% der Zeit gehen frs Anzeigen drauf, also gehen wir von 4MHZ aus
while (zeitms != 0) { //nicht sonderlich genau
    zeitms--;
    _delay_loop_2(F_CPU/4000);//8000 Takte = 1ms bei 8MHZ
  }
}

void error(u08 e_code) {
//Timer2 + USART-In Interrupts deaktivieren
TCCR2 = 0; //Timer 2 stoppen
UCSRB = 0; //UART deaktivieren
PORTD &= ~(1<<2);
//LEDs bliken mit dem Fehlercode
//Endlosschleife bis zum Reset
taster = 0;
for (;;) {
  leds = e_code;
  waitms(250);
  leds = 0;
  waitms(250);
  if (taster == 2) {
    //Watchdog fr Reset aktivieren
    taster = 0;
    WDTCR = 1<<WDE; //Watchdog enabled
  }
}
}

void fifo_error(u08 e_code) {
if (halt_on_err != 0) {
  error(e_code);
}
}
  
void control(void) {
if (mastermode == 1) {
  //Senden von regelmigen "Pings"
  if (ping_next == 0) {
    ping_next = 96; //alle 3sec
    send_funk_put(0x0110); //Sende Ping
  }
  if (nosend == 255) {
    nosend = 128; // in 4 sec darf gesendet wertden
  }
}
}

void syscommand(u08 value) {
if (value == 0x10) { //Jemand bittet uns zu antworten (ping)
  send_funk_put(0x0111); //Antwort auf Ping senden
}
if (value == 0x11) { //Antwort auf Ping erhalten
  connected_tx = 160; //fr 5sec
  if (mastermode == 1) {
    send_funk_put(0x0112); //Antwort auf Antwort
  }
}
if (value == 0x12) { //Antwort auf Antwort erhalten
  connected_tx = 160; //fr 5 sec
}
if (value == 0x20) { //Wechsel auf Slave
  mastermode = 0;
}
if (value == 0x21) { //Wechel auf Master
  mastermode = 1;
}

}

void movedata(void) {
u08 val;
u16 val2;
/* von rec_232 in send_funk, in send_funk nur,
 wenn wirklich leer um Platz fr Systembefehle zu lassen */
if ((rec_232_status() != 0) && (send_funk_free > (send_funk_size-4))) {
  val = rec_232_get();
  send_funk_put(val);
}
//von rec_funk in send_232
if ((rec_funk_status() != 0) && (send_232_status() != 2)) {
  val2 = rec_funk_get();
  val = (u08)val2;
  connected_rx = 160; //fr 5 sec
  if (val2 > 0xFF) { //systembefehl!
    syscommand(val);   
  } else {
    send_232_put(val);
  }
}
}


void sysinit(void) {
waitms(1000);
//I/O Init
DDRB = 0x07;
DDRC = 0x30;
DDRD = 0x3E;
//Pull-up
PORTB |= (1<<3);
//Timer0 Init
timer0_init();
//Interrupts global aktivieren
sei();
//Alle LEDs an
leds = 0xFF;
waitms(200);
//Taster
while (tast_wert != 0) { //warten auf Ende des Tastendrucks
  asm volatile ("nop");
}
if (taster > 2 ) error(1); //Ungltiger Tasterwert
if (taster == 2) eep_restoredefault();
taster = 0;
//Konfiguration
if (eep_loadconf() == 0) error(2); // 1 -> ok, 0-> fehler
//Timer2 Init
timer2_init();
//Sender + Empfnger Testen
send_funk_put(0x010F);//Systembefehl, 0x0F
if ((io_funk_send() != 0) && (halt_on_err != 0)) {
  error(3); //Keine Daten Empfangen, oder Empfnger bereits beschftigt
}
ad_init();
rs232int_init();
nosend = 255;
leds = 0;
}

int main(void) {
sysinit();
for(;;) {
  taster_act();
  if (interface_active == 0) {
    movedata();
  } else {
    interface();
  }
  io_funk_rec();
  led_update();
  io_funk_send();
  io_232_send();
  control();
}
}
